@@ -1,7 +1,7 @@
 /*
  * gauche.h - Gauche scheme system header
  *
- *   Copyright (c) 2000-2023  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -41,6 +41,9 @@
    instances. */
 #include <gauche/config.h>
 #include <gauche/config_threads.h>
+#if defined(LIBGAUCHE_BODY)
+#include "gauche/priv/configP.h" /* Only used while compiling Gauche itself */
+#endif
 
 /* GAUCHE_API_VERSION is GAUCHE_MAJOR_VERSION*1000 + revision.
    The revision is only incremented when we change API, which we expect
@@ -57,6 +60,10 @@
 //#define GAUCHE_API_VERSION 1000
 #endif
 
+/* As of 1.0, We always use utf8 in internal encoding.  However,
+   existing code may check this. */
+#define GAUCHE_CHAR_ENCODING_UTF8 1
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
@@ -69,7 +76,6 @@
 #include <errno.h>
 #include <stdint.h>
 #include <inttypes.h>
-#include <gauche/int64.h>
 
 #ifdef HAVE_SYS_TIME_H
 # include <sys/time.h>
@@ -148,14 +154,17 @@ SCM_DECL_BEGIN
 
 /* Include appropriate threading interface.  Threading primitives are
    abstracted with SCM_INTERNAL_* macros and ScmInternal* typedefs.
-   See gauche/uthread.h for the semantics of these primitives. */
+   See gauche/pthread.h for the semantics of these primitives. */
 #ifdef GAUCHE_USE_PTHREADS
 # include <gauche/pthread.h>
 #elif  GAUCHE_USE_WTHREADS
 # include <gauche/wthread.h>
-#else  /* !GAUCHE_USE_PTHREADS */
-# include <gauche/uthread.h>
-#endif /* !GAUCHE_USE_PTHREADS */
+#else
+# error "No thread support"
+#endif
+
+/* For the backward compatibility.  Now we always have threads. */
+#define GAUCHE_HAS_THREADS 1
 
 #define SCM_WORD_BITS   (SIZEOF_LONG*8)
 
@@ -419,15 +428,7 @@ SCM_EXTERN ScmChar Scm_CharDowncase(ScmChar ch);
 SCM_EXTERN ScmChar Scm_CharTitlecase(ScmChar ch);
 SCM_EXTERN ScmChar Scm_CharFoldcase(ScmChar ch);
 
-#if   defined(GAUCHE_CHAR_ENCODING_EUC_JP)
-#include "gauche/char_euc_jp.h"
-#elif defined(GAUCHE_CHAR_ENCODING_UTF_8)
 #include "gauche/char_utf_8.h"
-#elif defined(GAUCHE_CHAR_ENCODING_SJIS)
-#include "gauche/char_sjis.h"
-#else
-#include "gauche/char_none.h"
-#endif
 
 /* Character lexer category.  See 7.1.1 of R7RS */
 typedef enum {
@@ -1709,6 +1710,7 @@ SCM_EXTERN void Scm_Init(const char *signature);
 SCM_EXTERN int  Scm_InitializedP(void);
 SCM_EXTERN void Scm_Cleanup(void);
 SCM_EXTERN void Scm_Exit(int code) SCM_NORETURN;
+SCM_EXTERN int  Scm_ObjToExitCode(ScmObj obj);
 SCM_EXTERN void Scm_Abort(const char *msg) SCM_NORETURN;
 SCM_EXTERN void Scm_Panic(const char *msg, ...) SCM_NORETURN;
 
