@@ -1,7 +1,7 @@
 /*
  * exception.h - more exception classes
  *
- *   Copyright (c) 2000-2022  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -44,16 +44,18 @@
     |    +- <error>             ; srfi-35, also inherits <serious-condition>
     |         +- <system-error>
     |         +- <unhandled-signal-error>
+    |         +- <unbound-varaible-error>
     |         +- <read-error> ; srfi-36
     |         +- <io-error>   ; srfi-36
-    |              +- <port-error> ; srfi-36
-    |                   +- <io-read-error>   ; srfi-36
-    |                        +- <io-decoding-error>  ; (srfi-186)
-    |                   +- <io-write-error>  ; srfi-36
-    |                        +- <io-encoding-error>  ; (srfi-186)
-    |                   +- <io-closed-error> ; srfi-36
-    |                   +- <io-unit-error>
-    |                   +- <io-invalid-position-error> ; srfi-192
+    |         |    +- <port-error> ; srfi-36
+    |         |         +- <io-read-error>   ; srfi-36
+    |         |         |    +- <io-decoding-error>  ; (srfi-186)
+    |         |         +- <io-write-error>  ; srfi-36
+    |         |         |    +- <io-encoding-error>  ; (srfi-186)
+    |         |         +- <io-closed-error> ; srfi-36
+    |         |         +- <io-unit-error>
+    |         |         +- <io-invalid-position-error> ; srfi-192
+    |         +- <continuation-error>     ; srfi-226
     +- <thread-exception> ; srfi-18
     |    +- <join-timeout-exception>      ; srfi-18
     |    +- <abandoned-mutex-exception>   ; srfi-18
@@ -170,6 +172,19 @@ SCM_CLASS_DECL(Scm_UnhandledSignalErrorClass);
 #define SCM_UNHANDLED_SIGNAL_ERROR(obj)   ((ScmUnhandledSignalError*)(obj))
 #define SCM_UNHANDLED_SIGNAL_ERROR_P(obj) SCM_ISA(obj, SCM_CLASS_UNHANDLED_SIGNAL_ERROR)
 
+/* <unbound-variable-error>: unbound variable */
+typedef struct ScmUnboundVariableErrorRec {
+    ScmError common;
+    ScmObj identifier;          /* (</> <symbol> <identifier>) */
+} ScmUnboundVariableError;
+
+SCM_CLASS_DECL(Scm_UnboundVariableErrorClass);
+#define SCM_CLASS_UNBOUND_VARIABLE_ERROR  (&Scm_UnboundVariableErrorClass)
+#define SCM_UNBOUND_VARIABLE_ERROR(obj)   ((ScmUnboundVariableError*)(obj))
+#define SCM_UNBOUND_VARIABLE_ERROR_P(obj) SCM_ISA(obj, SCM_CLASS_UNBOUND_VARIABLE_ERROR)
+
+SCM_EXTERN ScmObj Scm_MakeUnboundVariableError(ScmObj identifier);
+
 /* <read-error>: error from the reader */
 typedef struct ScmReadErrorRec {
     ScmError common;
@@ -231,6 +246,16 @@ SCM_CLASS_DECL(Scm_IOEncodingErrorClass);
 SCM_CLASS_DECL(Scm_IOInvalidPositionErrorClass);
 #define SCM_CLASS_IO_INVALID_POSITION_ERROR  (&Scm_IOInvalidPositionErrorClass)
 
+/* <continuation-error> */
+typedef struct ScmContinuationErrorRec {
+    ScmError common;
+    ScmObj promptTag;
+} ScmContinuationError;
+
+SCM_CLASS_DECL(Scm_ContinuationErrorClass);
+#define SCM_CLASS_CONTINUATION_ERROR   (&Scm_ContinuationErrorClass)
+#define SCM_CONTINUATION_ERROR_P(obj)  SCM_ISA(obj, SCM_CLASS_CONTINUATION_ERROR)
+
 /*---------------------------------------------------
  * Compounders
  */
@@ -306,12 +331,39 @@ SCM_CLASS_DECL(Scm_UncaughtExceptionClass);
    since it won't be used for multiple inheritance; instead, an instance
    of the mixin class is compounded at runtime.  See, for example,
    the 'compile' function in compile.scm mixing <compile-error-mixin>
-   into the error raised during compilation.  */
+   into the error raised during compilation.
+
+   The default error reporter calls a method report-additional-condition
+   on each mixin condition in the error heading.
+   See print-default-error-heading in libexc.scm.  The method definitions
+   are in libomega.scm.
+ */
 
 SCM_CLASS_DECL(Scm_MixinConditionClass);
 #define SCM_CLASS_MIXIN_CONDITION   (&Scm_MixinConditionClass)
 #define SCM_MIXIN_CONDITION_P(obj)  SCM_ISA(obj, SCM_CLASS_MIXIN_CONDITION)
 
+/* <condition-continuation-mixin>
+   This attaches the continuation of the expression that raised it.
+   You can extract stack trace from it.
+   Some implementations attach stack trace to a condition unconditionally;
+   in Gauche it is not a lightweight operation to extract a continuation,
+   so we chose to let the code decide whether to attach this info.
+ */
+typedef struct ScmConditionContinuationMixinRec {
+    ScmCondition common;
+    ScmObj continuation;
+} ScmConditionContinuationMixin;
+
+SCM_CLASS_DECL(Scm_ConditionContinuationMixinClass);
+#define SCM_CLASS_CONDITION_CONTINUATION_MIXIN (&Scm_ConditionContinuationMixinClass)
+#define SCM_CONDITION_CONTINUATION_MIXIN_P(obj) SCM_ISA(obj, SCM_CLASS_CONDITION_CONTINUATION_MIXIN)
+#define SCM_CONDITION_CONTINUATION_MIXIN(obj) ((ScmConditionContinuationMixin*)(obj))
+
+/* <load-condition-mixin>
+   Attached to the error during loading.
+ */
+
 typedef struct ScmLoadConditionMixinRec {
     ScmCondition common;
     ScmObj history;             /* current-load-history */
@@ -323,6 +375,12 @@ SCM_CLASS_DECL(Scm_LoadConditionMixinClass);
 #define SCM_LOAD_CONDITION_MIXIN_P(obj) SCM_ISA(obj, SCM_CLASS_LOAD_CONDITION_MIXIN)
 #define SCM_LOAD_CONDITION_MIXIN(obj)  ((ScmLoadConditionMixin*)(obj))
 
+/* <compile-error-mixin>
+   Attached to the error during compiling.
+   The EXPR slot contains the offending expression.  If it is a pair,
+   you may extract source info from it.
+ */
+
 typedef struct ScmCompileErrorMixinRec {
     ScmCondition condition;
     ScmObj expr;                /* offending expr */
