@@ -1,7 +1,7 @@
 /*
  * vm.h - Virtual machine
  *
- *   Copyright (c) 2000-2023  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -261,6 +261,8 @@ SCM_CLASS_DECL(Scm_PromptTagClass);
 SCM_EXTERN ScmObj Scm_MakePromptTag(ScmObj name);
 SCM_EXTERN ScmObj Scm_DefaultPromptTag();
 
+typedef struct ScmContinuationPromptRec ScmContinuationPrompt;
+
 /*
  * Identifier
  *
@@ -452,14 +454,8 @@ struct ScmVMRec {
                                    is used only in compilation. */
     ScmCStack *cstack;          /* current escape point.  see the comment of
                                    "C stack rewinding" below. */
-    /* TRANSIENT: Change these flags to u_long in 1.0 */
-#if GAUCHE_API_VERSION < 98
-    unsigned int runtimeFlags;  /* Runtime flags */
-    unsigned int compilerFlags; /* Compiler flags */
-#else /*GAUCHE_API_VERSION >= 98*/
-    u_long runtimeFlags;  /* Runtime flags */
-    u_long compilerFlags; /* Compiler flags */
-#endif /*GAUCHE_API_VERSION >= 98*/
+    u_long runtimeFlags;        /* Runtime flags */
+    u_long compilerFlags;       /* Compiler flags */
     intptr_t attentionRequest;  /* Flag if VM needs to check signal queue,
                                    finalizer queue, or stop request.
                                    This flag can be turned on asynchronously.
@@ -576,6 +572,7 @@ struct ScmVMRec {
     ScmCodeCache *codeCache;
 
     /* for reset/shift */
+    ScmContinuationPrompt *currentPrompt;
     ScmObj resetChain;          /* list of reset information,
                                    where reset information is
                                    (delimited . <dynamic handlers chain>).
@@ -705,14 +702,6 @@ SCM_EXTERN ScmObj Scm_VMGetDynamicHandlers(void);
 SCM_EXTERN void   Scm_VMSetDynamicHandlers(ScmObj handlers);
 SCM_EXTERN void   Scm_VMFlushDynamicHandlers(void);
 
-#if GAUCHE_API_VERSION < 98
-/* Deprecated - not that useful */
-SCM_EXTERN ScmObj Scm_VMDynamicWindC(ScmSubrProc *before,
-                                     ScmSubrProc *body,
-                                     ScmSubrProc *after,
-                                     void *data);
-#endif
-
 /*
  * Runtime flags
  */
