@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.cgen.cise - C in S expression
 ;;;
-;;;   Copyright (c) 2004-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -32,10 +32,11 @@
 ;;;
 
 (define-module gauche.cgen.cise
-  (use srfi-13)
+  (use srfi.13)
   (use gauche.sequence)
   (use gauche.cgen.unit)
   (use gauche.cgen.literal)
+  (use gauche.cgen.type)
   (use gauche.experimental.lamb)
   (use util.match)
   (export cise-render cise-render-to-string cise-render-rec
@@ -459,9 +460,9 @@
         (match args
           [() '()]
           [(('... . _)) '("...")]       ;varargs
-          [((var . type))
+          [((var ':: type))
            `(,@(gen-arg-1 var type))]
-          [((var . type) . rest)
+          [((var ':: type) . rest)
            `(,@(gen-arg-1 var type) "," ,@(loop rest))]))))
 
   (define (gen-qualifiers quals) ; we might support more qualifiers in future
@@ -517,15 +518,17 @@
             (record-static name quals args ret-type))
           ;; no function implementation
           '()])]))
+  (define (arg-decls args)
+    (cgen-canonical-typed-var-list args 'ScmObj))
 
   (ensure-toplevel-ctx form env)
   (match form
     [(_ name (args ...) ':: ret-type . body)
-     (check-quals name '() (canonicalize-argdecl args) ret-type body)]
+     (check-quals name '() (arg-decls args) ret-type body)]
     [(_ name (args ...) [? type-symbol? ts] . body)
-     (check-quals name '() (canonicalize-argdecl args) (type-symbol-type ts) body)]
+     (check-quals name '() (arg-decls args) (type-symbol-type ts) body)]
     [(_ name (args ...) . body)
-     (check-quals name '() (canonicalize-argdecl args) 'ScmObj body)]))
+     (check-quals name '() (arg-decls args) 'ScmObj body)]))
 
 ;;------------------------------------------------------------
 ;; Global variable definition and typedef
@@ -589,18 +592,8 @@
     (ensure-toplevel-ctx form env)
     (ensure-stmt-or-toplevel-ctx form env))
 
-  (let* ([canon (car (canonicalize-vardecl (list (cdr form))))]
-         [var (car canon)]
-         [spec (cdr canon)])
-    (receive (type init-and-quals)
-        (match spec
-          [()         (values 'ScmObj '())]
-          [('::)      (errorf "invalid variable decl in ~s: (~s ~s)"
-                              (car form) var spec)]
-          [(':: type) (values type '())]
-          [(':: type . init-and-quals) (values type init-and-quals)]
-          [else (values 'ScmObj spec)])
-      (check-quals var type '() init-and-quals))))
+  (receive [var type init-and-quals] (canonicalize-typed-var-1 form)
+    (check-quals var type '() init-and-quals)))
 
 ;;------------------------------------------------------------
 ;; CPS transformation
@@ -644,7 +637,7 @@
     [(_ rvar expr vars . body)
      (let* ([tmp-cc (gensym "tmp_cc_")]
             [data (gensym "data")]
-            [closed (canonicalize-argdecl vars)]
+            [closed (cgen-canonical-typed-var-list vars 'ScmObj)]
             [cc-env (make-env 'toplevel '())])
        ;; NB: We want to check the # of closed variables is smaller
        ;; than SCM_CCONT_DATA_SIZE, but it's not available at runtime
@@ -669,8 +662,9 @@
                            ,@body)
                    `(let* ,(map-with-index
                             (^[i p]
-                              `(,(car p) :: ,(cdr p)
-                                (cast (,(cdr p)) (aref ,data ,i))))
+                              (match-let1 [var ':: type] p
+                                `(,var :: ,type
+                                       (cast (,type) (aref ,data ,i)))))
                             closed)
                       ,@body)))
              'toplevel)))
@@ -710,22 +704,17 @@
   (ensure-stmt-ctx form env)
   (match form
     [(_ vars . body)
-     (match (canonicalize-vardecl vars)
-       [((var . spec) ...)
+     (match (cgen-canonical-typed-var-list vars 'ScmObj)
+       [((var ':: type . maybe-init) ...)
         (let1 eenv (expr-env env)
           `(begin
-             ,@(map (^[var spec]
-                      (receive (type has-init? init)
-                          (match spec
-                            [()         (values 'ScmObj #f #f)]
-                            [('::)      (errorf "invalid variable decl in let* form: (~s ~s)" var spec)]
-                            [(init)     (values 'ScmObj #t init)]
-                            [(':: type) (values type #f #f)]
-                            [(':: type init) (values type #t init)])
-                        `(,(cise-render-typed-var type var env)
-                          ,@(cond-list [has-init? `("=",(render-rec init eenv))])
-                          ";")))
-                    var spec)
+             ,@(map (^[var type maybe-init]
+                      `(,(cise-render-typed-var type var env)
+                        ,@(cond-list
+                           [(pair? maybe-init)
+                            `("=",(render-rec (car maybe-init) eenv))])
+                        ";"))
+                    var type maybe-init)
              ,@(map (cut render-rec <> env) body)))]
        [_ (error "invalid variable decls in let* form:" form)])]
     ))
@@ -1029,6 +1018,14 @@
           item)]
     [(_ . other) (error "malformed .include:" form)]))
 
+;; [cise toplevel/stmt] .error TOKEN ...
+;;   c preprocessor #error directive
+(define-cise-macro (.error form env)
+  (ensure-stmt-or-toplevel-ctx form env)
+  (match form
+    [(_ item ...) `("#error " ,@($ intersperse " "
+                                   $ map (cut format "~s" <>) item))]))
+
 (define-cise-macro |#if| .if)           ;backward compat.
 
 ;;------------------------------------------------------------
@@ -1364,10 +1361,7 @@
     [('.union tag . rest)
      (render-struct-or-union "union" tag #f rest var env)]
     [('.function (args ...) rettype . rest)
-     (let1 rt (let1 vv (canonicalize-vardecl `(_ ,rettype))
-                 (unless (null? (cdr vv))
-                  (errorf "Invalid return type in ~s" typespec))
-                (caddar vv))
+     (let1 rt (canonicalize-return-type rettype typespec)
        `(,(cise-render-typed-var rt "" env)
          "("
          ,(if (null? rest)
@@ -1377,7 +1371,7 @@
          "("
          ,@($ intersperse ", "
               $ map (^.[(arg ':: type) (cise-render-typed-var type arg env)])
-              $ canonicalize-vardecl args)
+              $ cgen-canonical-typed-var-list args 'ScmObj)
          ")"))]
     [(x)
      `(,(x->string x) " " ,(cise-render-identifier var))]
@@ -1394,7 +1388,7 @@
                     ,@(map (^.[(member ':: type)
                                `(,(cise-render-typed-var type member env)
                                  "; ")])
-                           (canonicalize-vardecl fields))
+                           (canonicalize-field-decls fields))
                     "} ")]
                  [(not fields) '(" ")])
     ,(if (null? rest)
@@ -1404,67 +1398,30 @@
 (define (cise-render-identifier sym)
   (cgen-safe-name-friendly (x->string sym)))
 
-;; canonicalize-vardecl
-;;
-;; Given list of C variable declarations, possibly typed, returns a list
-;; ((var :: type [init]) ...)
-;;
-;; Allows var::type as (var :: type)
-;; and (var::type init) as (var :: type init)
-;;
-;; It is also used to expand the field list of struct and union.
-;;
-;; Example: (foo bar::int baz::(const char*) (boom::int 0))
-;;  => ((foo :: ScmObj) (bar :: int) (baz :: (const char*)) (boom :: int 0))
-;;
-;; Note: To avoid any type decl to be generated, give || as a type, e.g.
-;; "foo::||".  It is unusual, but needed if the variable name is actually
-;; a macro to be expanded into typed variable name.  SCM_HEADER is an example.
-;; We need a placeholder || to avoid the default type ScmObj is given.
-(define (canonicalize-vardecl vardecls)
-  (define (expand-type elt seed)
-    (cond
-     [(keyword? elt)  ;; The case of (var ::type)
-      (rxmatch-case (keyword->string elt)
-        [#/^:(.+)$/ (_ t) `(:: ,(string->symbol t) ,@seed)]
-        [else (cons elt seed)])]
-     [(symbol? elt)
-      (rxmatch-case (symbol->string elt)
-        [#/^([^:]+)::$/ (_ v) `(,(string->symbol v) :: ,@seed)]
-        [#/^([^:]+)::([^:]+)$/ (_ v t)
-            `(,(string->symbol v) :: ,(string->symbol t) ,@seed)]
-        [#/^([^:]+):(.*)$/ (#f #f #f) (err elt)]
-        [else (cons elt seed)])]
-     [else (cons elt seed)]))
+;; For field declaration, you may need to include a C macro that doesn't
+;; have type decl (e.g. SCM_HEADER).  If you just write SCM_HEADER,
+;; it generates 'ScmObj SCM_HEADER;' which you don't want.
+;  You can write SCM_HEADER::|| to suppress type decl.
+(define (canonicalize-field-decls fields)
+  (rlet1 fs (cgen-canonical-typed-var-list fields 'ScmObj)
+    (dolist [f fs]
+      (match f
+        [((? symbol?) ':: _ . _) #f]
+        [bad (error "Invalid field declaration: " bad)]))))
 
-  (define (err decl) (error "invalid variable declaration:" decl))
+;; Parse and get canonical form of return type.  Basically it's a
+;; typed var with missing variable, so we add a dummy var and parse it.
+(define (canonicalize-return-type rettype typespec)
+  (match (cgen-canonical-typed-var-list `(_ ,rettype) 'ScmObj)
+    [((_ ':: rt)) rt]
+    [_ (errorf "Invalid return type in ~s" typespec)]))
 
-  (define (scan in r)
-    (match in
-      [() (reverse r)]
-      [([? keyword? xx] . rest) (err xx)]
-      [([? symbol? var] ':: type . rest)
-       (scan rest `((,var :: ,type) ,@r))]
-      [([? symbol? var] . rest)
-       (scan rest `((,var :: ScmObj) ,@r))]
-      [(([? symbol? v] [? symbol? t] . args) . rest)
-       (scan rest `(,(expand-type v (expand-type t args)) ,@r))]
-      [(([? symbol? vt] . args) . rest)
-       (scan rest `(,(expand-type vt args) ,@r))]
-      [(xx . rest) (err xx)]))
-
-  (scan (fold-right expand-type '() vardecls) '()))
-
-;; Like canonicalize-vardecl, but for argument declarations.
-;; (foo::type bar baz:: type bee :: type)
-;; => ((foo . type) (bar . ScmObj) (baz . type) (bee . type))
-(define (canonicalize-argdecl argdecls)
-  (define (rec args)
-    (match (canonicalize-vardecl args)
-      [() '()]
-      [((var ':: type) . rest) `((,var . ,type) ,@(rec rest))]
-      [(var . rest) `((,var . ScmObj) ,@(rec rest))]))
-  (rec argdecls))
+;; Deal with define-cvar/declare-cvar/define-ctype form.
+;; Returns three values: variable, type, and a list of init value / qualifies
+(define (canonicalize-typed-var-1 form)
+  (match (cgen-canonical-typed-var-list (list (cdr form)) 'ScmObj)
+    [((var ':: type . init-and-quals)) (values var type init-and-quals)]
+    [z (error "Invalid variable decl in " z form)]))
 
 ;;=============================================================
 ;; Sealing the default environment
