@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.cgen.literal - static literal data
 ;;;
-;;;   Copyright (c) 2004-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -33,9 +33,11 @@
 
 (define-module gauche.cgen.literal
   (use scheme.list)
-  (use srfi-13)
-  (use srfi-42)
+  (use srfi.13)
+  (use srfi.42)
+  (use srfi.197)
   (use gauche.sequence)
+  (use gauche.threads)
   (use gauche.cgen.unit)
   (use util.match)
   (export <cgen-literal> cgen-c-name cgen-cexpr cgen-make-literal
@@ -394,27 +396,37 @@
   (rec (car key)))
 
 (define (literal-value=? x-key y-key)
-  (define (rec x y)
-    (cond
-     [(pair? x) (and (pair? y) (rec (car x) (car y)) (rec (cdr x) (cdr y)))]
-     [(vector? x)
-      (and (vector? y)
-           (let1 len (vector-length x)
-             (and (= len (vector-length y))
-                  (every?-ec (: i len)
-                             (rec (vector-ref x i) (vector-ref y i))))))]
-     [(or (uvector? x) (string? x) (char-set? x) (regexp? x)
-          (%descriptive-type? x))
-      (equal? x y)]
-     [(wrapped-identifier? x)
-      (and (wrapped-identifier? y)
-           (eq? (~ x'name) (~ y'name))
-           (eq? (~ x'module) (~ y'module)))]
-     ;; We can't use equal? as fallback, for the equality predicate needs
-     ;; to recurse to literal-value=?.
-     [else (and (eq? (class-of x) (class-of y)) (eqv? x y))]))
   (and (equal? (cdr x-key) (cdr y-key)) ;cpp conditions
-       (rec (car x-key) (car y-key))))
+       (literal-value=?-rec (car x-key) (car y-key))))
+
+;; NB: This should be a local function in literal-value=?, but 0.9.13 compiler
+;; fails to lift it, adding overhead of closure allocation.  Since
+;; literal-value=? is called many millions of times during compilation,
+;; we hand-optmize it for now.  Put it back once we have a better compiler.
+(define (literal-value=?-rec x y)
+  (cond
+   [(eqv? x y)]
+   [(pair? x) (and (pair? y)
+                   (literal-value=?-rec (car x) (car y))
+                   (literal-value=?-rec (cdr x) (cdr y)))]
+   [(vector? x)
+    (and (vector? y)
+         (let1 len (vector-length x)
+           (and (= len (vector-length y))
+                (let loop ([i 0])
+                  (cond [(= i len) #t]
+                        [(literal-value=?-rec (vector-ref x i) (vector-ref y i))
+                         (loop (+ i 1))]
+                        [else #f])))))]
+   [(or (uvector? x) (string? x) (char-set? x) (regexp? x)
+        (%descriptive-type? x))
+    (equal? x y)]
+   [(wrapped-identifier? x)
+    (and (wrapped-identifier? y)
+         (eq? (~ x'name) (~ y'name))
+         (eq? (~ x'module) (~ y'module)))]
+   [else #f]))
+
 
 (define (ensure-literal-hash unit)
   (or (~ unit'literals)
@@ -547,13 +559,13 @@
     (cond
      ;; We don't use fixnum?, since we may be cross-compiling on 64bit
      ;; machine for 32bit machine.  This is the range of 30bit fixnum.
-     [(< (- (%expt 2 29)) value (- (%expt 2 29) 1))
+     [(< (- (integer-expt 2 29)) value (- (integer-expt 2 29) 1))
       (make <cgen-scheme-integer> :value value :c-name #f)]
      ;; Integers that doesn't fit in 30bit fixnum but the literal number
      ;; can fit in C 32bit integers.  Note: We can use both signed and
      ;; unsigned literals, so the upper bound is 2^32-1, while
      ;; the lower bound is -2^31.
-     [(< (- (%expt 2 31)) value (- (%expt 2 32) 1))
+     [(< (- (integer-expt 2 31)) value (- (integer-expt 2 32) 1))
       (make <cgen-scheme-integer> :value value
             :c-name (cgen-allocate-static-datum))]
      [else
@@ -572,9 +584,9 @@
       ;; we can use 64bit literal, but we'll leave it for later revision.
       (let ([val   (~ self'value)]
             [cname (cgen-c-name self)])
-        (cond [(< (- (%expt 2 31)) val 0)
+        (cond [(< (- (integer-expt 2 31)) val 0)
                (print "  " cname " = Scm_MakeInteger("val");")]
-              [(<= 0 val (- (%expt 2 32) 1))
+              [(<= 0 val (- (integer-expt 2 32) 1))
                (print "  " cname " = Scm_MakeIntegerU("val"U);")]
               [else
                (print "  " cname " = Scm_StringToNumber(SCM_STRING("
@@ -743,10 +755,15 @@
     (let* ([value (~ self'value)]
            [class (class-of value)])
       (print "static "(uvector-class->c-type-name class)" "(~ self'elements)"[] = {")
-      (dotimes [i (uvector-length value)]
-        ($ uvector-class-emit-elt class
-           $ (with-module gauche.internal uvector-ref) value i))
-      (print  "};"))]
+      ;; Avoid using generic fold over uvector (value), for that would introduce
+      ;; dependency to gauche.uvector.
+      (do ([i 0 (+ i 1)]
+           [column 0 (uvector-class-emit-elt class
+                                             (uvector-ref value i)
+                                             column)])
+          [(= i (uvector-length value))]
+        )
+      (print "};"))]
   [static (self) #f]
   )
 
@@ -780,54 +797,67 @@
     [(eq? class <c128vector>) "ScmDoubleComplex"]
     ))
 
-(define (uvector-class-emit-elt class v)
-  (define (pr-half-float v)
-    (format #t "0x~4,'0x" ((with-module gauche.internal flonum->f16bits) v)))
-  (define (pr-float v)
-    (cond [(nan? v) (display "SCM_FLT_NAN")]
-          [(infinite? v) (display (if (> v 0)
-                                    "SCM_FLT_POSITIVE_INFINITY"
-                                    "SCM_FLT_NEGATIVE_INFINITY"))]
-          [(-zero? v) (display "-0.0f")]
-          [else (format #t "~sf" v)]))
-  (define (pr-double v)
-    (cond [(nan? v) (display "SCM_DBL_NAN")]
-          [(infinite? v) (display (if (> v 0)
-                                    "SCM_DBL_POSITIVE_INFINITY"
-                                    "SCM_DBL_NEGATIVE_INFINITY"))]
-          [(-zero? v) (display "-0.0")]
-          [else (format #t "~s" v)]))
+;; Returns a new column number (0-based)
+(define (uvector-class-emit-elt class v column)
+  (define (fmt-half-float v)
+    (format "0x~4,'0x" ((with-module gauche.internal flonum->f16bits) v)))
+  (define (fmt-float v)
+    (cond [(nan? v) "SCM_FLT_NAN"]
+          [(infinite? v) (if (> v 0)
+                           "SCM_FLT_POSITIVE_INFINITY"
+                           "SCM_FLT_NEGATIVE_INFINITY")]
+          [(negative-zero? v) "-0.0f"]
+          [else (format "~sf" v)]))
+  (define (fmt-double v)
+    (cond [(nan? v) "SCM_DBL_NAN"]
+          [(infinite? v) (if (> v 0)
+                           "SCM_DBL_POSITIVE_INFINITY"
+                           "SCM_DBL_NEGATIVE_INFINITY")]
+          [(negative-zero? v) "-0.0"]
+          [else (format "~s" v)]))
+  (define (fmt column fmtstr . args)
+    (let* ([s (apply format fmtstr args)]
+           [len (string-length s)])
+      (cond [(>= (+ column len) 70) (newline) (display s) len]
+            [else (display " ") (display s) (+ column len 1)])))
+  (define (prn column s)
+    (unless (zero? column) (newline))
+    (display s) (newline)
+    0)
   (cond
    [(memq class `(,<s8vector> ,<s16vector> ,<s32vector>))
-    (format #t "~d,\n" v)]
+    (fmt column "~d," v)]
    [(memq class `(,<u8vector> ,<u16vector> ,<u32vector>))
-    (format #t "~du,\n" v)]
+    (fmt column "~du," v)]
    [(eq? class <s64vector>)
-    (print "#if SIZEOF_LONG == 8")
-    (format #t "~dl,\n" v)
-    (print "#else  /*SIZEOF_LONG == 4*/")
-    (format #t " (((int64_t)~dl << 32)|~dl),\n"
-            (ash v -32) (logand v (- (%expt 2 32) 1)))
-    (print "#endif /*SIZEOF_LONG == 4*/")]
+    (chain (prn column "#if SIZEOF_LONG == 8")
+           (fmt _ " ~dl," v)
+           (prn _ "#else  /*SIZEOF_LONG == 4*/")
+           (fmt _ " (((int64_t)~dl << 32)|~dl),"
+                (ash v -32) (logand v (- (integer-expt 2 32) 1)))
+           (prn _ "#endif /*SIZEOF_LONG == 4*/"))]
    [(eq? class <u64vector>)
-    (print "#if SIZEOF_LONG == 8")
-    (format #t "~dlu,\n" v)
-    (print "#else  /*SIZEOF_LONG == 4*/")
-    (format #t " (((int64_t)~dlu << 32)|~dlu),\n"
-            (ash v -32) (logand v (- (%expt 2 32) 1)))
-    (print "#endif /*SIZEOF_LONG == 4*/")]
-   [(eq? class <f16vector>) (pr-half-float v) (print ",")]
-   [(eq? class <f32vector>) (pr-float v) (print ",")]
-   [(eq? class <f64vector>) (pr-double v) (print ",")]
+    (chain (prn column "#if SIZEOF_LONG == 8")
+           (fmt _ " ~dlu," v)
+           (prn _ "#else  /*SIZEOF_LONG == 4*/")
+           (fmt _ " (((int64_t)~dlu << 32)|~dlu),"
+                (ash v -32) (logand v (- (integer-expt 2 32) 1)))
+           (prn _ "#endif /*SIZEOF_LONG == 4*/"))]
+   [(eq? class <f16vector>) (fmt column "~a," (fmt-half-float v))]
+   [(eq? class <f32vector>) (fmt column "~a," (fmt-float v))]
+   [(eq? class <f64vector>) (fmt column "~a," (fmt-double v))]
    [(eq? class <c32vector>)
-    (display "{ ") (pr-half-float (real-part v))
-    (display ", ") (pr-half-float (imag-part v)) (print "},")]
+    (fmt column "{ ~a, ~a },"
+         (fmt-half-float (real-part v))
+         (fmt-half-float (imag-part v)))]
    [(eq? class <c64vector>)
-    (pr-float (real-part v)) (display " + ")
-    (pr-float (imag-part v)) (print " * _Complex_I,")]
+    (fmt column "~a + ~a * _Complex_I,"
+         (fmt-float (real-part v))
+         (fmt-float (imag-part v)))]
    [(eq? class <c128vector>)
-    (pr-double (real-part v)) (display " + ")
-    (pr-double (imag-part v)) (print " * _Complex_I,")]
+    (fmt column "~a + ~a * _Complex_I,"
+         (fmt-double (real-part v))
+         (fmt-double (imag-part v)))]
    ))
 
 ;; char-set -----------------------------------------------------
@@ -1005,6 +1035,42 @@
   (static (self) #f)
   )
 
+;; box ----------------------------------------------------------
+
+(define-cgen-literal <cgen-box> <box>
+  ((vals           :init-keyword :vals))
+  (make (value)
+    (make <cgen-box> :value value
+          :c-name (cgen-allocate-static-datum)
+          :vals (map cgen-literal (values->list (unbox value)))))
+  (init (self)
+    (let ([c-vals (map cgen-cexpr (~ self'vals))]
+          [cname (~ self'c-name)])
+      (if (length=? c-vals 1)
+        (format #t "  ~a = SCM_OBJ(Scm_MakeBox(~a));\n" cname (car c-vals))
+        (format #T "  ~a = SCM_OBJ(Scm_ListToMVBox(~a));\n"
+                cname (string-join c-vals ", ")))))
+  (static (self) #f)
+  )
+
+;; mutex --------------------------------------------------------
+
+;; Literal mutex is useful when a macro wants to insert a global lock.
+;; TRANSIENT: To compile 1.0 with 0.9.13, which doesn't export <mutex>,
+;; we need a global-variable-ref hack.  Remove it after 1.0 release.
+(define-cgen-literal <cgen-mutex> (global-variable-ref 'gauche.threads '<mutex>)
+  ((name           :init-keyword :name))
+  (make (value)
+    (make <cgen-mutex> :value value
+          :c-name (cgen-allocate-static-datum)
+          :name (cgen-literal (mutex-name value))))
+  (init (self)
+    (let ([name (cgen-cexpr (~ self'name))]
+          [cname (~ self'c-name)])
+      (format #t "  ~a = Scm_MakeMutex(~a);\n"
+              cname name)))
+  (static (self) #f)
+  )
 
 ;;---------------------------------------------------------------
 ;; Inferring literal handlers.
@@ -1013,8 +1079,8 @@
 ;;  constructor is defined.
 
 ;;  NB: this routine writes value to a string and read it back (sans
-;;  srfi-10 prefix if any).   If the written out value contains *nested*
-;;  srfi-10 syntax, reader constructor of the inner srfi-10 is invoked
+;;  SRFI-10 prefix if any).   If the written out value contains *nested*
+;;  SRFI-10 syntax, reader constructor of the inner srfi-10 is invoked
 ;;  in this process.  If the reader constructor have side effects, this
 ;;  routine may cause undesirable side effects as well.  There's no way
 ;;  we can know if it is safe.
@@ -1038,7 +1104,7 @@
 
 (define (infer-literal-handler value)
   (and-let* ([s (guard (e [else #f]) (write-to-string value))]
-             [m (#/#,\(/ s)]  ; srfi-10 syntax
+             [m (#/#,\(/ s)]  ; SRFI-10 syntax
              [xpr (guard (e [else #f]) (read-from-string (string-drop s 2)))])
     (match xpr
       [(tag . args)
