@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.cgen.precomp - Precompile Scheme into C data
 ;;;
-;;;   Copyright (c) 2004-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -36,11 +36,12 @@
 
 (define-module gauche.cgen.precomp
   (use scheme.list)
-  (use srfi-13)
+  (use srfi.13)
   (use gauche.cgen)
   (use gauche.cgen.stub)
   (use gauche.cgen.tmodule)
   (use gauche.vm.insn)
+  (use gauche.vm.debug-info)
   (use gauche.sequence)
   (use gauche.experimental.lamb)
   (use file.util)
@@ -169,10 +170,13 @@
                                :key (ext-initializer #f)
                                     ((:strip-prefix prefix) #f)
                                     ((:dso-name dso) #f)
+                                    (omit-line-directives #f)
+                                    ((:omit-debug-source-info no-source) #f)
                                     (predef-syms '())
                                     (macros-to-keep '())
                                     (single-sci-file #f)
                                     (load-paths '())
+                                    (target-parameters '())
                                     (extra-optimization #f))
   (define (precomp-1 main src)
     (let* ([out.c (cgen-scm-path->c-file (strip-prefix src prefix))]
@@ -181,12 +185,15 @@
       (%cgen-precompile src
                         :out.c out.c
                         :dso-name (or dso (basename-sans-extension main))
+                        :omit-line-directives omit-line-directives
+                        :omit-debug-source-info no-source
                         :predef-syms predef-syms
                         :strip-prefix prefix
                         :macros-to-keep macros-to-keep
                         :extra-optimization extra-optimization
                         :ext-initializer (and (equal? src main)
                                               ext-initializer)
+                        :target-parameters target-parameters
                         :load-paths load-paths
                         :initializer-name initname)))
   (define (tweak-sci sci) ; for consolidating sci files. returns sexp list
@@ -224,6 +231,8 @@
 (define (%cgen-precompile src
                           :key (out.c #f)
                                (out.sci #f)
+                               (omit-line-directives #f)
+                               ((:omit-debug-source-info no-source) #f)
                                ((:strip-prefix prefix) #f)
                                (ext-initializer #f)
                                ((:dso-name dso) #f)
@@ -232,9 +241,12 @@
                                (predef-syms '())
                                (load-paths '())
                                (macros-to-keep '())
+                               ((:target-parameters tparams) '())
                                (extra-optimization #f))
   (define (do-it)
-    (parameterize ([omitted-code '()])
+    (parameterize ([omitted-code '()]
+                   [omit-debug-source-info no-source]
+                   [target-parameters tparams])
       (setup ext-initializer sub-initializers)
       (with-input-from-file src
         (^[]
@@ -248,6 +260,8 @@
                  (reverse (generator-fold compile-toplevel-form '() read))))
             (set! *load-path* load-path-save))))
       (finalize sub-initializers)
+      (setup-debug-info)
+      (show-debug-info-stat (cgen-current-unit))
       (cgen-emit-c (cgen-current-unit))))
   (let ([out.c   (or out.c (path-swap-extension (sys-basename src) "c"))]
         [out.sci (or out.sci
@@ -263,6 +277,7 @@
                                        (basename-sans-extension out.c)))
                               initializer-name)]
                    [vm-eval-situation SCM_VM_COMPILING]
+                   [cise-emit-source-line (not omit-line-directives)]
                    [private-macros-to-keep macros-to-keep]
                    [run-extra-optimization-passes extra-optimization])
       (select-tmodule 'gauche)
@@ -316,8 +331,10 @@
   (eval expr (~ (current-tmodule)'module)))
 
 ;; Expr -> CompiledCode
+;;  target-params parameter is passed to 'compile'.
 (define (compile-in-current-tmodule expr)
-  (compile expr (~ (current-tmodule)'module)))
+  (compile expr (~ (current-tmodule)'module)
+           :target-params (target-parameters)))
 
 ;;================================================================
 ;; Parameters
@@ -353,6 +370,12 @@
 ;; Experimental: Run extra optimization during AOT compilation.
 (define run-extra-optimization-passes (make-parameter #f))
 
+;; Compile target parameters.  See 'compile' entry in compile.scm
+(define target-parameters (make-parameter '(:cont-frame-size 7)))
+
+;; Do not emit debug-source-info with the compiled code
+(define omit-debug-source-info (make-parameter #f))
+
 ;;================================================================
 ;; Bridge to the internal stuff
 ;;
@@ -402,7 +425,7 @@
 
 ;; Read the first form.
 ;; We don't read the entire content of the file, since it may contain
-;; srfi-10 read-time constructor that we don't know about yet.
+;; SRFI-10 read-time constructor that we don't know about yet.
 (define (first-form src) (with-input-from-file src read))
 
 ;; Check if the first form is define-module.
@@ -667,7 +690,9 @@
            filename
            (sys-dirname (port-name (current-input-port))))
       (^[]
-        (port-case-fold-set! (current-input-port) case-fold?)
+        (set! ((with-module gauche.internal port-case-fold)
+               (current-input-port))
+              case-fold?)
         (unwind-protect
             (generator-fold compile-toplevel-form seed read)
           (close-port (current-input-port))))))
@@ -870,6 +895,7 @@
    [code-vector-c-name :init-keyword :code-vector-c-name]
    [literals           :init-keyword :literals]
    [signature-info     :init-keyword :signature-info]
+   [debug-info-cname   :init-keyword :debug-info-cname]
    )
   (make (value)
     (let* ([code (if (run-extra-optimization-passes)
@@ -886,13 +912,14 @@
                 (cgen-safe-comment (~ code'name)))
         (format #t "            (ScmWord*)(~a), ~a,\n"
                 cvn (length cv))
-        (format #t "            ~a, ~a, ~a, ~a, SCM_NIL, ~a,\n"
+        (format #t "            ~a, ~a, ~a, ~a, ~a, ~a,\n"
                 (~ code'max-stack)
                 (~ code'required-args)
                 (~ code'optional-args)
                 (if (cgen-literal-static? code-name)
                   (cgen-cexpr code-name)
                   "SCM_FALSE")
+                "SCM_NIL"
                 (if (cgen-literal-static? signature-info)
                   (cgen-cexpr signature-info)
                   "SCM_FALSE"))
@@ -910,6 +937,8 @@
             :code-vector-c-name cvn
             :code-name code-name
             :signature-info signature-info
+            :debug-info-cname (and (not (omit-debug-source-info))
+                                   (serializable-debug-info code))
             :literals lv)))
   (init (self)
     (unless (cgen-literal-static? (~ self'code-name))
@@ -919,6 +948,9 @@
     (unless (cgen-literal-static? (~ self'signature-info))
       (print "  SCM_COMPILED_CODE("(~ self'c-name)")->signatureInfo = "
              (cgen-cexpr (~ self'signature-info)) ";"))
+    (when (~ self'debug-info-cname)
+      (print "  SCM_COMPILED_CODE("(~ self'c-name)")->debugInfo = "
+             (~ self'debug-info-cname) ";"))
     (fill-code self))
   (static (self) #t)
   )
@@ -940,6 +972,21 @@
             (unwrap-syntax (cdr sig)))
       (unwrap-syntax sig))))
 
+;; Construct serializable debug info.
+;;   See code.c Scm_CompiledCodePushInfo for the format of debug-info.
+(define (serializable-debug-info code)
+  (and-let1 di (~ code'debug-info)
+    (let1 codevec (encode-debug-info (cgen-current-unit)
+                                     (unwrap-syntax di))
+      (record-debug-info-stat! (cgen-current-unit) codevec #f)
+      (let ([codevec-lit (cgen-literal codevec)]
+            [datum-cname (cgen-allocate-static-datum 'runtime)])
+        (cgen-init (format "  ~a = Scm_MakePackedDebugInfo(SCM_U8VECTOR(~a),\
+                                         SCM_debug_info_const_vector);"
+                           datum-cname
+                           (cgen-cexpr codevec-lit)))
+        datum-cname))))
+
 ;; Returns a list of the same length of CODE, which includes the
 ;; <cgen-literal>s corresponding to the literal values in CODE.
 ;; #f is filled in the places that don't have corresponding litaral value.
@@ -1044,6 +1091,29 @@
                          il insns)])
            (cgen-literal packed)))))
 
+;; Debug info generation.  This must be called once, after all compiled codes
+;; are processed.
+(define (setup-debug-info)
+  (when (has-unit-debug-info? (cgen-current-unit))
+    (let* ([cvec (get-debug-info-const-vector (cgen-current-unit))]
+           [vec-literal (cgen-literal (make-vector (vector-length cvec) #f))]
+           [vec-name (cgen-cexpr vec-literal)]
+           [elt-literals (map cgen-literal cvec)])
+      (cgen-decl "static ScmObj SCM_debug_info_const_vector();")
+      (cgen-body "ScmObj SCM_debug_info_const_vector()"
+                 "{"
+                 ;; This is idempotent.  No need to mutex.
+                 "  static _Bool initialized = FALSE;"
+                 "  if (!initialized) {"
+                 "    int i = 0;")
+      (dolist [e elt-literals]
+        (cgen-body #"    SCM_VECTOR_ELEMENT(~|vec-name|, i++) = \
+                           ~(cgen-cexpr e);"))
+      (cgen-body "    initialized = TRUE;"
+                 "  }"
+                 #"  return ~|vec-name|;"
+                 "}"))))
+
 ;;----------------------------------------------------------------
 ;; <macro>
 ;;
