@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.cgen.type - Stub type management
 ;;;
-;;;   Copyright (c) 2004-2023  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -34,8 +34,10 @@
 (define-module gauche.cgen.type
   (use srfi.13)
   (use text.tr)
+  (use util.match)
   (use gauche.mop.instance-pool)
   (export <cgen-type> cgen-type-from-name make-cgen-type
+          cgen-canonical-typed-var-list
           cgen-boxer-name cgen-unboxer-name cgen-pred-name
           cgen-box-expr cgen-box-tail-expr cgen-unbox-expr cgen-pred-expr
           cgen-type-maybe? cgen-return-stmt)
@@ -79,7 +81,7 @@
 ;;
 ;; In general, types defined in extensions can't be directly accesible from
 ;; other extensions at C level.  So we don't need to carry around stub types
-;; across boundary of extensions.  Each extension is only accessible to
+;; across boundary of extensions.  Each extension can only use
 ;; the stub types provided from Gauche core, or the stub types explicitly
 ;; defined within it.
 ;; The stub types provided from Gauche core are defined in this module.
@@ -119,7 +121,7 @@
    ))
 
 (define-method write-object ((ct <cgen-type>) port)
-  (format port "#<cten-type ~a>" (~ ct'name)))
+  (format port "#<cgen-type ~a>" (~ ct'name)))
 
 ;; Lookup/create a cgen type from Gauche type name
 (define (cgen-type-from-name name)
@@ -208,9 +210,8 @@
   (%native <ssize_t> "Scm_IntegerFitsSsizeP" "Scm_IntegerToSsize" "Scm_SsizeToInteger")
   (%native <ptrdiff_t> "Scm_IntegerFitsPtrdiffP"  "Scm_IntegerToPtrdiff" "Scm_PtrdiffToInteger")
   (%native <off_t>   "Scm_IntegerFitsOffsetP" "Scm_IntegerToOffset" "Scm_OffsetToInteger")
-  ;; TRANSIENT: The following needs to be in after 0.9.13 release
-  ;;(%native <intptr_t>   "Scm_IntegerFitsIntptrP" "Scm_IntegerToIntptr" "Scm_IntptrToInteger")
-  ;;(%native <uintptr_t>   "Scm_IntegerFitsUintptrP" "Scm_IntegerToUintptr" "Scm_UintptrToInteger")
+  (%native <intptr_t>   "Scm_IntegerFitsIntptrP" "Scm_IntegerToIntptr" "Scm_IntptrToInteger")
+  (%native <uintptr_t>   "Scm_IntegerFitsUintptrP" "Scm_IntegerToUintptr" "Scm_UintptrToInteger")
 
   (%native <closure> "SCM_CLOSUREP" "SCM_CLOSURE" "SCM_OBJ")
   (%native <void>    "" "" "SCM_VOID_RETURN_VALUE")
@@ -297,8 +298,10 @@
    (%cclass <compiled-code> "ScmCompiledCode*")
    (%cclass <foreign-pointer> "ScmForeignPointer*")
    (%cclass <box>  "ScmBox*")
-   ;(%cclass <thread-local> "ScmThreadLocal*")
+   (%cclass <thread-local> "ScmThreadLocal*")
    (%cclass <primitive-parameter> "ScmPrimitiveParameter*")
+   (%cclass <parameterization> "ScmParameterization*")
+   (%cclass <continuation-mark-set> "ScmContinuationMarkSet*")
    (%cclass <dlobj> "ScmDLObj*")
    (%cclass <dlptr> "ScmObj" "Scm_DLPtrP" "SCM_OBJ")
    (%cclass <sys-group> "ScmSysGroup*")
@@ -309,11 +312,13 @@
    ;(%cclass <sys-tm> "ScmSysTm*")
    (%cclass <sys-fdset> "ScmSysFdset*")
 
-   ;; Exception - These classes are not available until we load gauche.threads,
-   ;; but we need those stub types before compiling gauche.threads.
-   ;; We hand-wire them, leaving scheme-type field #f.
-   (make-cgen-type '<mutex> #f "ScmMutex*" "<mutex>")
-   (make-cgen-type '<condition-variable> #f "ScmConditionVariable*"
+   ;; Exception - those classes are in the core but defined in gauche.threads.
+   (make-cgen-type '<mutex>
+                   (with-module gauche.threads <mutex>)
+                   "ScmMutex*" "<mutex>")
+   (make-cgen-type '<condition-variable>
+                   (with-module gauche.threads <condition-variable>)
+                   "ScmConditionVariable*"
                    "<condition-variable>")
    )
 
@@ -354,3 +359,95 @@
 
 (define (cgen-return-stmt expr)
   #"SCM_RETURN(~expr);")
+
+;;;
+;;; Parsing typed variables.
+;;;
+
+;
+;; For conciseness and readability, we allow varaible name and type
+;; can be concatenated, so the following forms are allowed:
+;;
+;;    var :: type        ; proper form
+;;    var::type          ; concatenated to a single symbol
+;;    var:: type         ; type can be S-expr
+;;    var ::type         ; non standard, but allowed
+;;
+;; Typed variables usually comes with list, something like this:
+;;
+;;   (a b::<int> c::(<List> <integer>) :optional (d::<int> 3)
+;;
+;; Canonicalization regognizes variations and produces this:
+;;
+;;   ((a :: default-type)
+;;    (b :: <int>)
+;;    (c :: (<List> <integer>))
+;;    :optional (d :: <int> 3))
+;;
+;; Each typed variable becomes this form:
+;;
+;;   (varname :: type [init-val] [qual ...])
+;;
+;; If type is omitted, <top> is assumed for stubs, while ScmObj is
+;; assumed for CiSEs, so the default-type is provided by the caller.
+;;
+;; This is used in muliple places with slightly different requirements.
+;;
+;;  1. Stubs
+;;    1a. Parsing argument list of cproc
+;;    1b. Parsing argument list of cmethod
+;;    1c. Parsing return value types of them
+;;
+;;  2. CiSE
+;;    1a. Parsing argument list of cfn
+;;    1b. Parsing return value types of cfn
+;;    1c. Parsing varaible declaration of let*
+;;    1d. Parsing struct/union declaration
+;;
+
+;; API
+;;   Take a list of typed var list, canonicalize the first one,
+;;   returns its canonical form and the rest of the input list.
+;;   Canonical form may be #f if the tip of the typed-var-list is
+;;   not a typed var.
+(define (cgen-canonical-typed-var-list typed-var-list
+                                       default-type)
+  ;; Split shorthand notations
+  (define (expand-type elt seed)
+    (cond
+     [(keyword? elt)
+      (rxmatch-case (keyword->string elt)
+        [#/^:(.+)$/ (_ t) `(:: ,(string->symbol t) ,@seed)]
+        [else (cons elt seed)])]
+     [(symbol? elt)
+      (rxmatch-case (symbol->string elt)
+        [#/^([^:]+)::$/ (_ v) `(,(string->symbol v) :: ,@seed)]
+        [#/^([^:]+)::([^:]+)$/ (_ v t)
+            `(,(string->symbol v) :: ,(string->symbol t) ,@seed)]
+        [#/^([^:]+):(.*)$/ (#f #f #f) (err elt)]
+        [else (cons elt seed)])]
+     [else (cons elt seed)]))
+
+  (define (err decl) (error "invalid variable declaration:" decl))
+
+  (define (scan in r)
+    (match in
+      [() (reverse r)]
+      [([? keyword? xx] . rest) (reverse r (cons xx rest))]
+      [([? symbol? var] ':: type . rest)
+       (scan rest `((,var :: ,type) ,@r))]
+      [([? symbol? var] . rest)
+       (scan rest `((,var :: ,default-type) ,@r))]
+      [(([? symbol? v] [? symbol? t] . args) . rest)
+       (let1 sub (expand-type v (expand-type t args))
+         (match sub
+           [(_ ':: . _) (scan rest `(,sub ,@r))]
+           [(var . opt) (scan rest `((,var :: ,default-type ,@opt) ,@r))]))]
+      [(([? symbol? vt] . args) . rest)
+       (let1 sub (expand-type vt args)
+         (match sub
+           [(_ ':: . _) (scan rest `(,sub ,@r))]
+           [(var . opt) (scan rest `((,var :: ,default-type ,@opt) ,@r))]))]
+      [(xx . rest) (reverse r (cons xx rest))]))
+
+  (scan (fold-right expand-type '() typed-var-list) '()))
