@@ -1,7 +1,7 @@
 ;;;
 ;;; configure.scm - configuring Gauche extensions
 ;;;
-;;;   Copyright (c) 2013-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2013-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -84,8 +84,8 @@
   (use file.util)
   (use text.tr)
   (use text.tree)
-  (use srfi-13)
-  (use srfi-113) ; sets & bags
+  (use srfi.13)
+  (use scheme.set)
   (extend gauche.config)
   (export cf-init cf-init-gauche-extension
           cf-arg-enable cf-arg-with cf-feature-ref cf-package-ref
@@ -207,8 +207,7 @@
                            (bug-report #f) (url #f))
   (when (current-package)
     (exit 1 "Incorrect configure script: cf-init is called more than once."))
-  (let* ([gpd (and-let* ([srcdir (current-load-path)]
-                         [pfile (build-path (sys-dirname srcdir) "package.scm")]
+  (let* ([gpd (and-let* ([pfile (build-path 'cld "package.scm")]
                          [ (file-exists? pfile) ])
                 (path->gauche-package-description pfile))]
          ;; We need to support old way (package info given to cf-init) and
@@ -285,6 +284,7 @@
     (parse-command-line-arguments)
     (check-directory-names)
     (process-args)
+    (check-package-superseded-by)
     (check-requirements)))
 
 (define (initialize-default-definitions)
@@ -361,9 +361,9 @@
   (cf-subst 'OBJEXT (gauche-config "--object-suffix"))
   (cf-subst 'EXEEXT (gauche-config "--executable-suffix"))
 
-  ;; '-DUNICODE' if gauche is in utf8 and on windows.
+  ;; '-DUNICODE' on windows.
   (cf-subst 'WINDOWS_UNICODE_FLAG
-            (cond-expand [(and gauche.os.windows gauche.ces.utf8) "-DUNICODE"]
+            (cond-expand [gauche.os.windows "-DUNICODE"]
                          [else ""]))
   )
 
@@ -482,6 +482,14 @@
   (cf-subst 'top_builddir (cf$ 'builddir))
   )
 
+(define (check-package-superseded-by)
+  ;; If package.scm has superseded-by clause, tell so.
+  (and-let1 sup (~ (current-package)'gpd'superseded-by)
+    (cf-msg-error "This package is superseded by ~a and no longer maintained. \
+                   Use the successor module."
+                  sup))
+  )
+
 ;; Process the arg-if-given and arg-if-not-given callbacks of cf-arg-*.
 (define (process-args)
   (dolist [entry (arg-processors)]
@@ -1027,7 +1035,7 @@
                      '("FILE *f = fopen(\"conftest.out\", \"w\");\n"
                        "return ferror(f) || fclose(f) != 0;")))
 
-(define-method cf-lang-call-m ((lang <c-language>) prologue body)
+(define-method cf-lang-call-m ((lang <c-language>) prologue func-name)
   ($ cf-lang-program-m lang
      (if (equal? func-name "main")
        `(,prologue
