@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.test - test framework
 ;;;
-;;;   Copyright (c) 2000-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -84,8 +84,10 @@
 (define-module gauche.test
   (export test test* test-start test-end test-running? test-section test-log
           test-module test-script test*/diff
-          test-error test-one-of test-none-of
+          test-error test-one-of test-none-of test-truthy
           test-check test-check-diff
+          test-include-r7
+
           test-report-failure test-report-failure-plain
           test-report-failure-diff
           test-record-file test-summary-check
@@ -99,6 +101,7 @@
 (autoload "gauche/test/script" test-script)
 (autoload "gauche/test/diff"
           (:macro test*/diff) test-check-diff test-report-failure-diff)
+(autoload "gauche/test/include" (:macro test-include-r7))
 
 ;; An object to represent error.  This class isn't exported; the user
 ;; must use `test-error' procedure to create an instance.
@@ -147,6 +150,14 @@
 ;; API
 (define (test-none-of . choices) (make <test-none-of> :choices choices))
 
+;; An object to represent "any true value"
+(define-class <test-truthy> () ())
+(define-method write-object ((obj <test-truthy>) out)
+  (format out "#<test-truthy>"))
+
+;; API
+(define (test-truthy) (make <test-truthy>))
+
 ;; API
 ;; We don't use generic function dispatch (at least for the time being),
 ;; to make it easy to troubleshoot when object system gets messed up.
@@ -169,6 +180,7 @@
         [(is-a? expected <test-none-of>)
          (every (lambda (choice) (not (test-check choice result fallback)))
                 (slot-ref expected 'choices))]
+        [(is-a? expected <test-truthy>) result]
         [else (fallback expected result)]))
 
 (define *test-error* (make <test-error>)) ;DEPRECATED
@@ -561,7 +573,7 @@
     (if (null? *discrepancy-list*)
       (fmt "passed.\n")
       (begin
-        (fmt "failed.\ndiscrepancies found.  Errors are:\n")
+        (fmt "failed.\n~a discrepanc~:*~@p found:\n" (length *discrepancy-list*))
         (for-each (lambda (r)
                     (apply (lambda (report msg expect actual)
                              (fmt "test ~a: expects ~s => got ~a\n" msg expect
