@@ -1,7 +1,7 @@
 ;;;
 ;;; http.scm - HTTP 1.1
 ;;;
-;;;   Copyright (c) 2000-2023  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -737,7 +737,7 @@
             [(eq? (~ conn'secure) 'tls)
              (cond-expand
               [gauche.net.tls
-               (tls-connect (make-tls :server-name (~ conn'server)
+               (tls-connect (make-tls :server-name host
                                       :options (if (tls-ca-bundle-path)
                                                  0
                                                  SSL_SERVER_VERIFY_LATER))
