@@ -1,7 +1,7 @@
 ;;;
 ;;; cgi.scm - CGI utility
 ;;;
-;;;   Copyright (c) 2000-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -39,7 +39,7 @@
 
 (define-module www.cgi
   (use scheme.list)
-  (use srfi-13)
+  (use srfi.13)
   (use rfc.uri)
   (use rfc.mime)
   (use rfc.822)
